@@ -104,6 +104,16 @@ class TestFindAll(TreeTest):
         self.assertSelects(soup('a', limit=1), ["1"])
         self.assertSelects(soup.b(id="foo"), ["3"])
 
+    def test_find_all_with_self_referential_data_structure_does_not_cause_infinite_recursion(self):
+        soup = self.soup("<a></a>")
+        # Create a self-referential list.
+        l = []
+        l.append(l)
+
+        # Without special code in _normalize_search_value, this would cause infinite
+        # recursion.
+        self.assertEqual([], soup.find_all(l))
+
 class TestFindAllBasicNamespaces(TreeTest):
 
     def test_find_by_namespaced_name(self):
@@ -192,6 +202,14 @@ class TestFindAllByAttribute(TreeTest):
         self.assertSelects(tree.find_all(id='first'),
                            ["Matching a.", "Matching b."])
 
+    def test_find_all_by_utf8_attribute_value(self):
+        peace = u"םולש".encode("utf8")
+        data = u'<a title="םולש"></a>'.encode("utf8")
+        soup = self.soup(data)
+        self.assertEqual([soup.a], soup.find_all(title=peace))
+        self.assertEqual([soup.a], soup.find_all(title=peace.decode("utf8")))
+        self.assertEqual([soup.a], soup.find_all(title=[peace, "something else"]))
+
     def test_find_all_by_attribute_dict(self):
         # You can pass in a dictionary as the argument 'attrs'. This
         # lets you search for attributes like 'name' (a fixed argument
@@ -210,18 +228,24 @@ class TestFindAllByAttribute(TreeTest):
         self.assertSelects(tree.find_all(attrs={'name' : 'name1'}),
                            ["Name match."])
 
-        # Passing class='class2' would cause a syntax error.
         self.assertSelects(tree.find_all(attrs={'class' : 'class2'}),
                            ["Class match."])
 
     def test_find_all_by_class(self):
-        # Passing in a string to 'attrs' will search the CSS class.
         tree = self.soup("""
                          <a class="1">Class 1.</a>
                          <a class="2">Class 2.</a>
                          <b class="1">Class 1.</b>
                          <c class="3 4">Class 3 and 4.</c>
                          """)
+
+        # Passing in the class_ keyword argument will search against
+        # the 'class' attribute.
+        self.assertSelects(tree.find_all('a', class_='1'), ['Class 1.'])
+        self.assertSelects(tree.find_all('c', class_='3'), ['Class 3 and 4.'])
+        self.assertSelects(tree.find_all('c', class_='4'), ['Class 3 and 4.'])
+
+        # Passing in a string to 'attrs' will also search the CSS class.
         self.assertSelects(tree.find_all('a', '1'), ['Class 1.'])
         self.assertSelects(tree.find_all(attrs='1'), ['Class 1.', 'Class 1.'])
         self.assertSelects(tree.find_all('c', '3'), ['Class 3 and 4.'])
@@ -230,17 +254,15 @@ class TestFindAllByAttribute(TreeTest):
     def test_find_by_class_when_multiple_classes_present(self):
         tree = self.soup("<gar class='foo bar'>Found it</gar>")
 
-        attrs = { 'class' : re.compile("o") }
-        f = tree.find_all("gar", attrs=attrs)
+        f = tree.find_all("gar", class_=re.compile("o"))
         self.assertSelects(f, ["Found it"])
 
-        f = tree.find_all("gar", re.compile("a"))
+        f = tree.find_all("gar", class_=re.compile("a"))
         self.assertSelects(f, ["Found it"])
 
         # Since the class is not the string "foo bar", but the two
         # strings "foo" and "bar", this will not find anything.
-        attrs = { 'class' : re.compile("o b") }
-        f = tree.find_all("gar", attrs=attrs)
+        f = tree.find_all("gar", class_=re.compile("o b"))
         self.assertSelects(f, [])
 
     def test_find_all_with_non_dictionary_for_attrs_finds_by_class(self):
@@ -265,8 +287,9 @@ class TestFindAllByAttribute(TreeTest):
         self.assertEqual([a, a2], soup.find_all("a", "foo"))
         self.assertEqual([a], soup.find_all("a", "bar"))
 
-        # If you specify the attribute as a string that contains a
+        # If you specify the class as a string that contains a
         # space, only that specific value will be found.
+        self.assertEqual([a], soup.find_all("a", class_="foo bar"))
         self.assertEqual([a], soup.find_all("a", "foo bar"))
         self.assertEqual([], soup.find_all("a", "bar foo"))
 
@@ -352,7 +375,7 @@ class TestFindAllByAttribute(TreeTest):
 class TestIndex(TreeTest):
     """Test Tag.index"""
     def test_index(self):
-        tree = self.soup("""<wrap>
+        tree = self.soup("""<div>
                             <a>Identical</a>
                             <b>Not identical</b>
                             <a>Identical</a>
@@ -360,10 +383,10 @@ class TestIndex(TreeTest):
                             <c><d>Identical with child</d></c>
                             <b>Also not identical</b>
                             <c><d>Identical with child</d></c>
-                            </wrap>""")
-        wrap = tree.wrap
-        for i, element in enumerate(wrap.contents):
-            self.assertEqual(i, wrap.index(element))
+                            </div>""")
+        div = tree.div
+        for i, element in enumerate(div.contents):
+            self.assertEqual(i, div.index(element))
         self.assertRaises(ValueError, tree.index, 1)
 
 
@@ -712,11 +735,11 @@ class TestTreeModification(SoupTest):
         new_a = a.replace_with(soup.c)
         self.assertEqual(a, new_a)
 
-    def test_replace_with_children_returns_thing_that_was_replaced(self):
+    def test_unwrap_returns_thing_that_was_replaced(self):
         text = "<a><b></b><c></c></a>"
         soup = self.soup(text)
         a = soup.a
-        new_a = a.replace_with_children()
+        new_a = a.unwrap()
         self.assertEqual(a, new_a)
 
     def test_replace_tag_with_itself(self):
@@ -821,6 +844,18 @@ class TestTreeModification(SoupTest):
         self.assertEqual(the.next_element, c_tag)
         self.assertEqual(c_tag.previous_element, the)
 
+    def test_append_child_thats_already_at_the_end(self):
+        data = "<a><b></b></a>"
+        soup = self.soup(data)
+        soup.a.append(soup.b)
+        self.assertEqual(data, soup.decode())
+
+    def test_move_tag_to_beginning_of_parent(self):
+        data = "<a><b></b><c></c><d></d></a>"
+        soup = self.soup(data)
+        soup.a.insert(0, soup.d)
+        self.assertEqual("<a><d></d><b></b><c></c></a>", soup.decode())
+
     def test_insert_works_on_empty_element_tag(self):
         # This is a little strange, since most HTML parsers don't allow
         # markup like this to come through. But in general, we don't
@@ -882,6 +917,18 @@ class TestTreeModification(SoupTest):
         self.assertEqual(no.next_element, "no")
         self.assertEqual(no.next_sibling, " business")
 
+    def test_replace_first_child(self):
+        data = "<a><b></b><c></c></a>"
+        soup = self.soup(data)
+        soup.b.replace_with(soup.c)
+        self.assertEqual("<a><c></c></a>", soup.decode())
+
+    def test_replace_last_child(self):
+        data = "<a><b></b><c></c></a>"
+        soup = self.soup(data)
+        soup.c.replace_with(soup.b)
+        self.assertEqual("<a><b></b></a>", soup.decode())
+
     def test_nested_tag_replace_with(self):
         soup = self.soup(
             """<a>We<b>reserve<c>the</c><d>right</d></b></a><e>to<f>refuse</f><g>service</g></e>""")
@@ -918,14 +965,35 @@ class TestTreeModification(SoupTest):
         self.assertEqual(g_tag.previous_element, to_text)
         self.assertEqual(g_tag.previous_sibling, to_text)
 
-    def test_replace_with_children(self):
+    def test_unwrap(self):
         tree = self.soup("""
             <p>Unneeded <em>formatting</em> is unneeded</p>
             """)
-        tree.em.replace_with_children()
+        tree.em.unwrap()
         self.assertEqual(tree.em, None)
         self.assertEqual(tree.p.text, "Unneeded formatting is unneeded")
 
+    def test_wrap(self):
+        soup = self.soup("I wish I was bold.")
+        value = soup.string.wrap(soup.new_tag("b"))
+        self.assertEqual(value.decode(), "<b>I wish I was bold.</b>")
+        self.assertEqual(
+            soup.decode(), self.document_for("<b>I wish I was bold.</b>"))
+
+    def test_wrap_extracts_tag_from_elsewhere(self):
+        soup = self.soup("<b></b>I wish I was bold.")
+        soup.b.next_sibling.wrap(soup.b)
+        self.assertEqual(
+            soup.decode(), self.document_for("<b>I wish I was bold.</b>"))
+
+    def test_wrap_puts_new_contents_at_the_end(self):
+        soup = self.soup("<b>I like being bold.</b>I wish I was bold.")
+        soup.b.next_sibling.wrap(soup.b)
+        self.assertEqual(2, len(soup.b.contents))
+        self.assertEqual(
+            soup.decode(), self.document_for(
+                "<b>I like being bold.I wish I was bold.</b>"))
+
     def test_extract(self):
         soup = self.soup(
             '<html><body>Some content. <div id="nav">Nav crap</div> More content.</body></html>')
@@ -990,6 +1058,16 @@ class TestTreeModification(SoupTest):
         soup.b.string = "bar"
         self.assertEqual(soup.b.contents, ["bar"])
 
+    def test_string_set_does_not_affect_original_string(self):
+        soup = self.soup("<a><b>foo</b><c>bar</c>")
+        soup.b.string = soup.c.string
+        self.assertEqual(soup.a.encode(), b"<a><b>bar</b><c>bar</c></a>")
+
+    def test_set_string_preserves_class_of_string(self):
+        soup = self.soup("<a></a>")
+        cdata = CData("foo")
+        soup.a.string = cdata
+        self.assertTrue(isinstance(soup.a.string, CData))
 
 class TestElementObjects(SoupTest):
     """Test various features of element objects."""
@@ -1215,6 +1293,23 @@ class TestSubstitutions(SoupTest):
             decoded,
             self.document_for(u"<b><FOO></b><b>BAR</b>"))
 
+    def test_formatter_is_run_on_attribute_values(self):
+        markup = u'<a href="http://a.com?a=b&c=é">e</a>'
+        soup = self.soup(markup)
+        a = soup.a
+
+        expect_minimal = u'<a href="http://a.com?a=b&amp;c=é">e</a>'
+
+        self.assertEqual(expect_minimal, a.decode())
+        self.assertEqual(expect_minimal, a.decode(formatter="minimal"))
+
+        expect_html = u'<a href="http://a.com?a=b&amp;c=&eacute;">e</a>'
+        self.assertEqual(expect_html, a.decode(formatter="html"))
+
+        self.assertEqual(markup, a.decode(formatter=None))
+        expect_upper = u'<a href="HTTP://A.COM?A=B&C=É">E</a>'
+        self.assertEqual(expect_upper, a.decode(formatter=lambda x: x.upper()))
+
     def test_prettify_accepts_formatter(self):
         soup = BeautifulSoup("<html><body>foo</body></html>")
         pretty = soup.prettify(formatter = lambda x: x.upper())
@@ -1241,10 +1336,8 @@ class TestSubstitutions(SoupTest):
                     'http-equiv="Content-type"/>')
         soup = self.soup(meta_tag)
 
-        # Parse the document, and the charset is replaced with a
-        # generic value.
-        self.assertEqual(soup.meta['content'],
-                          'text/html; charset=%SOUP-ENCODING%')
+        # Parse the document, and the charset apprears unchanged.
+        self.assertEqual(soup.meta['content'], 'text/html; charset=x-sjis')
 
         # Encode the document into some encoding, and the encoding is
         # substituted into the meta tag.
@@ -1297,6 +1390,24 @@ class TestEncoding(SoupTest):
         self.assertRaises(
             UnicodeEncodeError, soup.encode, "ascii", errors="strict")
 
+    def test_decode_contents(self):
+        html = u"<b>\N{SNOWMAN}</b>"
+        soup = self.soup(html)
+        self.assertEqual(u"\N{SNOWMAN}", soup.b.decode_contents())
+
+    def test_encode_contents(self):
+        html = u"<b>\N{SNOWMAN}</b>"
+        soup = self.soup(html)
+        self.assertEqual(
+            u"\N{SNOWMAN}".encode("utf8"), soup.b.encode_contents(
+                encoding="utf8"))
+
+    def test_deprecated_renderContents(self):
+        html = u"<b>\N{SNOWMAN}</b>"
+        soup = self.soup(html)
+        self.assertEqual(
+            u"\N{SNOWMAN}".encode("utf8"), soup.b.renderContents())
+
 class TestNavigableStringSubclasses(SoupTest):
 
     def test_cdata(self):
@@ -1309,6 +1420,24 @@ class TestNavigableStringSubclasses(SoupTest):
         self.assertEqual(soup.find(text="foo"), "foo")
         self.assertEqual(soup.contents[0], "foo")
 
+    def test_cdata_is_never_formatted(self):
+        """Text inside a CData object is passed into the formatter.
+
+        But the return value is ignored.
+        """
+
+        self.count = 0
+        def increment(*args):
+            self.count += 1
+            return "BITTER FAILURE"
+
+        soup = self.soup("")
+        cdata = CData("<><><>")
+        soup.insert(1, cdata)
+        self.assertEqual(
+            b"<![CDATA[<><><>]]>", soup.encode(formatter=increment))
+        self.assertEqual(1, self.count)
+
     def test_doctype_ends_in_newline(self):
         # Unlike other NavigableString subclasses, a DOCTYPE always ends
         # in a newline.
@@ -1455,7 +1584,6 @@ class TestSoupSelector(TreeTest):
         self.assertSelects('.s1 > a', ['s1a1', 's1a2'])
         self.assertSelects('.s1 > a span', ['s1a2s1'])
 
-
     def test_attribute_equals(self):
         self.assertSelectMultiple(
             ('p[class="onep"]', ['p1']),
